package uk.gov.dwp.maze;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

public class Explorer {

    private static Logger LOG = Logger.getLogger(Explorer.class.getName());
    private Maze maze;
    private List<Character> route = new ArrayList<Character>();
    private Position currentPosition;
    private Compass compass = new Compass();


    public Explorer(Maze maze) {
        this.maze = maze;
        this.currentPosition = maze.getStart();
        LOG.info("Start is (" + currentPosition.getX() + "," + currentPosition.getY() + ")");
    }

    public void turnLeft(){
        route.add(Movement.LEFT.value());
        compass.rotateLeft();
    }

    public void turnRight(){
        route.add(Movement.RIGHT.value());
        compass.rotateRight();
    }

    public void moveForward(){
        if(canNextMoveToPosition()){
            route.add(Movement.FORWARD.value());
            currentPosition = currentPosition.getNextForwardPosition(getCurrentDirection());
            LOG.info("Moved to ("+currentPosition.getX()+","+currentPosition.getY()+")");
        }
    }

    public void moveForward(int k) {
        for(int i =0; i  < k; i++){
            moveForward();
        }
    }

    public Direction getCurrentDirection(){
        return compass.getCurrentDirection();
    }

    public boolean canNextMoveToPosition(){
        Position nextPosition = currentPosition.getNextForwardPosition(getCurrentDirection());;
        if (maze.isWallAt(nextPosition.getX(),nextPosition.getY())) {
            LOG.info("There's a wall at (" + nextPosition.getX() + "," + nextPosition.getY() + ")");
            return false;
        }
        return true;
    }

    public List<Character> getRoute() {
        return route;
    }




    public boolean isAtFinish() {
        return maze.isFinish(currentPosition);
    }

    private static class Compass {

        private Direction[][] points = new Direction[][]{
                {Direction.NORTH,Direction.EAST},
                {Direction.WEST,Direction.SOUTH}
        };

        public Direction getCurrentDirection(){
            return points[0][0];
        }

        public void rotateLeft(){
            Direction n = points[0][0];
            points[0][0] = points[1][0];
            points[1][0] = points[1][1];
            points[1][1] = points[0][1];
            points[0][1] = n;
        }

        public void rotateRight(){
            Direction n = points[0][0];
            points[0][0] = points[0][1];
            points[0][1] = points[1][1];
            points[1][1] = points[1][0];
            points[1][0] = n;

        }

    }
}

