package uk.gov.dwp.maze;


public class Maze {

    public static char FINISH = 'F';
    public static char START = 'S';
    public static char WALL = 'X';
    public static char SPACE = ' ';
    public Position start;
    public Position finish;

    private int numberOfWalls;
    private int numberOfSpaces;

    protected char[][] arr;

    public Maze(char[][] grid) {
        this.arr = grid;
        validateMaze();
    }

    public Maze(String[] strings) {
        this(createFromLines(strings));
    }

    public boolean isWallAt(int x, int y) {
        return getCharAt(x, y) == Maze.WALL;
    }

    public boolean isFinishAt(int x, int y) {
        return getCharAt(x,y) == Maze.FINISH;
    }

    public Position getStart() {
        return start;
    }

    public Position getFinish() {
        return finish;
    }

    public int getNumberOfWalls() {
        return numberOfWalls;
    }

    public int getNumberOfSpaces() {
        return numberOfSpaces;
    }

    private void validateMaze() {
        boolean foundStart = false;
        boolean foundExit = false;
        for (int y = 0; y < arr.length; y++) {
            for (int x = 0; x < arr[y].length; x++) {
                char c = arr[y][x];
                if (c == FINISH) {
                    if (foundExit) {
                        throw new IllegalArgumentException("Maze with multiple finish points not allowed");
                    } else {
                        finish = new Position(x,y);
                        foundExit = true;
                    }
                } else if (c == START) {
                    if (foundStart) {
                        throw new IllegalArgumentException("Maze with multiple start points not allowed");
                    } else {
                        start = new Position(x,y);
                        foundStart = true;
                    }
                } else if (c == WALL) {
                    numberOfWalls++;
                } else if (c == SPACE) {
                    numberOfSpaces++;
                } else {
                    throw new IllegalArgumentException("Maze is not properly defined");
                }
            }
        }
        if (!foundStart) throw new IllegalArgumentException("Maze doesn't have a start");
        if (!foundExit) throw new IllegalArgumentException("Maze doesn't have a finish");
    }

    private char getCharAt(int x, int y) {
        return arr[y][x];
    }

    private static char[][] createFromLines(String[] lines) {
        char[][] grid = new char[lines.length][lines[0].toCharArray().length];
        for (int x = 0; x < lines.length; x++) {
            char[] cArr = lines[x].toCharArray();
            for (int y = 0; y < cArr.length; y++) {
                grid[x][y] = cArr[y];
            }
        }
        return grid;
    }


    public boolean isFinish(Position currentPosition) {
        return finish.equals(currentPosition);
    }
}

