package uk.gov.dwp.maze;

import org.hamcrest.Matchers;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.List;

import static org.hamcrest.Matchers.contains;
import static org.junit.Assert.*;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class ExplorerTest {

    @Mock
    private Maze m1;

    @Before
    public void startUp(){
        when(m1.getStart()).thenReturn(new Position(3,3));
    }

    @Test
    public void testTurningAndMovingAddsExpectedRoute(){
        Explorer explorer = new Explorer(m1);
        assertTrue("Route should be empty to start with", explorer.getRoute().isEmpty());
        when(m1.isWallAt(2, 3)).thenReturn(false);
        when(m1.isWallAt(1, 3)).thenReturn(true);
        explorer.turnLeft();
        assertThat(explorer.getRoute(), contains('L'));
        explorer.turnLeft();
        assertThat(explorer.getRoute(), contains('L', 'L'));
        explorer.turnRight();
        assertThat(explorer.getRoute(), contains('L', 'L', 'R'));
        explorer.moveForward();
        assertThat(explorer.getRoute(), contains('L', 'L', 'R', 'F'));
        explorer.moveForward(); //can;t move forward because 1,3 is a wall
        assertThat(explorer.getRoute(), contains('L', 'L', 'R', 'F'));
    }

    @Test
    public void testGettingCurrentDirectionAfterTurnLeft(){
        Explorer explorer = new Explorer(m1);
        explorer.turnLeft();
        assertEquals(Direction.WEST, explorer.getCurrentDirection());
        explorer.turnLeft();
        assertEquals(Direction.SOUTH, explorer.getCurrentDirection());
        explorer.turnLeft();
        assertEquals(Direction.EAST, explorer.getCurrentDirection());
        explorer.turnLeft(); //rotated full  360
        assertEquals(Direction.NORTH, explorer.getCurrentDirection());
    }

    @Test
    public void testGettingCurrentDirectionAfterTurnRight(){
        Explorer explorer = new Explorer(m1);
        explorer.turnRight();
        assertEquals(Direction.EAST, explorer.getCurrentDirection());
        explorer.turnRight();
        assertEquals(Direction.SOUTH, explorer.getCurrentDirection());
        explorer.turnRight();
        assertEquals(Direction.WEST, explorer.getCurrentDirection());
        explorer.turnRight(); //rotated full  360
        assertEquals(Direction.NORTH, explorer.getCurrentDirection());;
    }

    @Test
    public void testCanReachEndOfMazeAndGetCorrectRoute(){
        Maze realMaze = new Maze(MazeTest.MAZE_FIXTURE);
        assertEquals(new Position(3,3),realMaze.getStart());
        assertEquals(new Position(1,14),realMaze.getFinish());
        Explorer explorer = new Explorer(realMaze);
        explorer.turnRight();
        explorer.moveForward(8);
        explorer.turnRight();
        explorer.moveForward(3);
        explorer.turnRight();
        explorer.moveForward(5);
        explorer.turnLeft();
        explorer.moveForward(3);
        explorer.turnRight();
        explorer.moveForward(3);
        explorer.turnLeft();
        explorer.moveForward(3);
        explorer.turnLeft();
        explorer.moveForward(8);
        explorer.turnRight();
        explorer.moveForward(1);
        explorer.turnLeft();
        explorer.moveForward(2);
        explorer.turnLeft();
        explorer.moveForward(12);
        explorer.turnLeft();
        explorer.moveForward(12);
        explorer.turnLeft();
        explorer.moveForward(13);
        assertTrue("Didn't reach finish",explorer.isAtFinish());
        String routeAsString = convertToString(explorer.getRoute());
        String expectedRoute = "RFFFFFFFFRFFFRFFFFFLFFFRFFFLFFFLFFFFFFFFRFLFFLFFFFFFFFFFFFLFFFFFFFFFFFFLFFFFFFFFFFFFF";
        assertEquals(expectedRoute,routeAsString);
    }

    private String convertToString(List<Character> route) {
        StringBuilder b = new StringBuilder();
        for(Character c : route){
            b.append(c);
        }
        return b.toString();
    }


}
